#include <SoftwareSerial.h>
/**
 * Use this sketch for point-point(P2P) communication of LoRa modules.
 * This sketch is for receiving messages.
 **/

SoftwareSerial loraSerial(2, 3);
#define debugSerial Serial

#define HEX_CHAR_TO_NIBBLE(c) ((c >= 'A') ? (c - 'A' + 0x0A) : (c - '0'))
#define HEX_PAIR_TO_BYTE(h, l) ((HEX_CHAR_TO_NIBBLE(h) << 4) + HEX_CHAR_TO_NIBBLE(l))

void setup() {
  while(!debugSerial || !loraSerial);

  debugSerial.begin(115200);
  loraSerial.begin(57600);

  //clear buffer just in case
  loraSerial.flush();
  
  // use only LoRa stack
  sendCommand("mac pause");
  
  /* uncomment this 2 lines if you want to use LoRaWAN stack
   For this case you should be using LoRaWAN library*/
  // sendCommand("mac resume");
  // while(1); 
  
  //Set 0 timeout for receiving messages
  sendCommand("radio set wdt 0");

  //listen for incoming messages regardless of length
  sendCommand("radio rx 0");
}

void loop() {
  
  if(loraSerial.available() > 0){
    //read whole response
    String s = loraSerial.readStringUntil("\r\n");

    //filter the response and get payload
    int lastIndex = s.indexOf("\r\n");
    String filtered = s.substring(10, lastIndex);

    //display the payload
    debugSerial.print("Payload: ");

    byte payload[filtered.length()/2];

    for(int i=0;i<filtered.length()/2;i++){
      payload[i] = HEX_PAIR_TO_BYTE(filtered[i*2], filtered[i*2+1]);
      debugSerial.print(payload[i]);
      debugSerial.print(" ");
    }
    debugSerial.println();
    
    //listen for next messages
    sendCommand("radio rx 0");
  }
}

void sendCommand(const char* str){
  //sending command
  loraSerial.println(str);
  
 //read the response
  String s = loraSerial.readStringUntil('\n'); 
  //print out the response
  debugSerial.print("Response: ");
  debugSerial.println(s);
}

